@@ -4,12 +4,13 @@ use anyhow::{anyhow, bail, Context, Result};
 use lazy_static::lazy_static;
 use std::{
   collections::{HashMap, HashSet},
+  fmt::Debug,
   io::Write,
   process::Command,
 };
 use tempfile::NamedTempFile;
 
-use flowistry::{Config, Range, SliceOutput};
+use flowistry::{Direction, FunctionIdentifier, Range};
 
 fn parse_ranges(
   prog: &str,
@@ -113,11 +114,15 @@ fn compare_ranges(expected: HashSet<Range>, actual: HashSet<Range>, prog: &str)
     }
   };
 
-  check(missing, "Missing");
-  check(extra, "Extra");
+  check(missing, "Actual was missing");
+  check(extra, "Actual contained unexpected");
 }
 
-pub fn flow(prog: &str, qpath: &str) {
+pub fn flow<O: Debug>(
+  prog: &str,
+  id: FunctionIdentifier,
+  cb: impl FnOnce(FunctionIdentifier, &[String]) -> Result<O>,
+) {
   let inner = move || -> Result<()> {
     let mut f = NamedTempFile::new()?;
     let _filename = f.path().to_string_lossy().to_string();
@@ -130,7 +135,7 @@ pub fn flow(prog: &str, qpath: &str) {
     );
     let args = args.split(" ").map(|s| s.to_owned()).collect::<Vec<_>>();
 
-    let output = flowistry::flow(qpath.to_owned(), &args);
+    let output = cb(id, &args);
     println!("{:?}", output.unwrap());
 
     Ok(())
@@ -139,7 +144,7 @@ pub fn flow(prog: &str, qpath: &str) {
   inner().unwrap();
 }
 
-pub fn slice(prog: &str, cb: impl FnOnce(Config, &[String]) -> Result<SliceOutput>) {
+pub fn slice(prog: &str, direction: Direction) {
   let inner = move || -> Result<()> {
     let mut f = NamedTempFile::new()?;
     let filename = f.path().to_string_lossy().to_string();
@@ -151,11 +156,6 @@ pub fn slice(prog: &str, cb: impl FnOnce(Config, &[String]) -> Result<SliceOutpu
 
     f.as_file_mut().write(prog_clean.as_bytes())?;
 
-    let config = Config {
-      range: range.clone(),
-      ..Default::default()
-    };
-
     let args = format!(
       "--edition=2018 --crate-name tmp {} -A warnings --sysroot {}",
       f.path().display(),
@@ -164,7 +164,7 @@ pub fn slice(prog: &str, cb: impl FnOnce(Config, &[String]) -> Result<SliceOutpu
 
     let args = args.split(" ").map(|s| s.to_owned()).collect::<Vec<_>>();
 
-    let output = cb(config, &args)?;
+    let output = flowistry::slice(direction, range, &args)?;
     let actual = output.ranges().into_iter().cloned().collect::<HashSet<_>>();
 
     compare_ranges(expected, actual, &prog_clean);
@@ -176,11 +176,19 @@ pub fn slice(prog: &str, cb: impl FnOnce(Config, &[String]) -> Result<SliceOutpu
 }
 
 pub fn backward_slice(prog: &str) {
-  slice(prog, flowistry::backward_slice);
+  slice(prog, Direction::Backward);
 }
 
 pub fn forward_slice(prog: &str) {
-  slice(prog, flowistry::forward_slice);
+  slice(prog, Direction::Forward);
+}
+
+pub fn effects(prog: &str, qpath: &str) {
+  flow(
+    prog,
+    FunctionIdentifier::Qpath(qpath.to_owned()),
+    flowistry::effects,
+  );
 }
 
 lazy_static! {
